from hashlib import sha256
from .utils import random_string, normalize


async def is_authenticated(token, db):
    return await db.tokens.find_one({"token": token}) is not None


async def register(uname, pwd, db):
    uname = normalize(uname)
    if (await db.credentials.find_one({"uname": uname})) is not None:
        return False, 'Already Exists'
    else:
        salt = random_string(100)
        hsh = sha256((pwd + salt).encode()).hexdigest()
        data = {"uname": uname, 'salt': salt, 'hsh': hsh}
        await db.credentials.insert_one(data)
        return True, 'ok'


async def generate_token(uname, pwd, db):
    uname = normalize(uname)
    data = await db.credentials.find_one({"uname": uname})
    if data is None:
        return False, 'No such user'
    hsh = sha256((pwd + data['salt']).encode()).hexdigest()
    if hsh != data['hsh']:
        return False, 'Wrong PWD'
    token = random_string()
    await db.tokens.insert_one({"token": token, "uname": uname})
    return True, token


async def remove_token(token, db):
    await db.tokens.find_one_and_delete({"token": token})


async def get_user_from_token(token, db):
    u = await db.tokens.find_one({"token": token})
    if u is None:
        return None
    return u['uname']



from time import sleep
from .core import Attempt, Question, TestCase
import pymongo
import random


def _attempt_checker(mongo_uri):
    db = pymongo.MongoClient(mongo_uri)
    db = db.openjudge
    while True:
        attempt = db.attempt_queue.find_one_and_delete({})
        if attempt is None:
            sleep(random.random())
        else:
            att = Attempt()
            att.__dict__ = attempt
            # has this been answered correctly earlier?
            query = {"qid": att.qid, "user": att.user,
                     "status": True}
            if db.history.find_one(query) is None:
                q = db.questions.find_one({"qid": att.qid})
                que = Question()
                que.__dict__ = q
                que.test_cases = [TestCase(**t) for t in q['test_cases']]
                checked_attempt = que(att)
            else:
                att.status = False
                att.log = None
                checked_attempt = att
            db.history.insert_one(checked_attempt.__dict__)


def run_judge(mongo_uri, n_judges):
    _attempt_checker(mongo_uri)



import os
import pkgutil


def __copy_template(templatedir):
    if not os.path.exists(templatedir):
        html = pkgutil.get_data('openjudge', 'templates/home.html')
        if html is not None:
            os.mkdir(templatedir)
            with open(os.path.join(templatedir, 'home.html'), 'w') as fl:
                fl.write(html.decode())


def __copy_static(staticdir):
    if not os.path.exists(staticdir):
        os.mkdir(staticdir)
        for st in ['Chart.min.js', 'jquery-3.2.1.min.js',
                   'js.cookie.min.js', 'main.css', 'main.js',
                   'showdown.min.js']:
            content = pkgutil.get_data('openjudge', 'staticfiles/'+st)
            if content is not None:
                with open(os.path.join(staticdir, st), 'w') as fl:
                    fl.write(content.decode())


def __copy_wrapper(wrapperfile):
    if not os.path.exists(wrapperfile):
        wrap = pkgutil.get_data('openjudge', 'wrappers.json')
        if wrap is not None:
            with open(wrapperfile, 'w') as fl:
                fl.write(wrap.decode())


def copy_defaults(tempdir, staticdir, wrapperfile):
    __copy_wrapper(wrapperfile)
    __copy_template(tempdir)
    __copy_static(staticdir)
